# Common settngs across all Makefiles
#
TOPDIR ?= ../..
LIBDIR = $(TOPDIR)/lib
CONFIGDIR = $(TOPDIR)/config

# Default job name.  Used when developing scripts
B ?= gcc-native

REVNO = $(shell bzr revno $(TOPDIR))

REVISION = cbuild$(REVNO)

# Pick the foo^variant suffix out of the target
VARIANT = $(word 2, $(subst ^, ,$(B)^))

LIMIT ?= 

LATEST_TOOLCHAIN = gcc-linaro-4.6-2012.11
BUILD = build

TOP := $(shell pwd)
MACHINE := $(shell uname -m)
# Host name.  Use /etc as this may be a chroot
HOST := $(shell cat /etc/hostname)
CODENAME ?= $(shell lsb_release -sc)

NPROCESSORS := $(shell getconf _NPROCESSORS_ONLN)
PARALLEL ?= -j$(NPROCESSORS)
CHECK_PARALLEL ?= $(PARALLEL)

-include $(TOPDIR)/siterc
-include local.mk
-include $(HOME)/.config/cbuild/cbuildrc

# Pull in any overrides from the job itself
-include $(B).job
# Pull in the overrides from a possible top level job file.  Used for
# jobs like the benchmarks that spawn under a different name.
-include $(BTOP).job

CONFIG ?= $(MACHINE)

-include $(CONFIGDIR)/$(VARIANT).mk
include $(CONFIGDIR)/$(CONFIG).mk

# The name of the configuration being built
# The last word in MAKEFILE_LIST is the last-parsed makefile.  Strip this down to the name.
CONFIG_NAME_AUTO := $(subst .mk,,$(notdir $(lastword $(MAKEFILE_LIST))))
CONFIG_NAME ?= $(CONFIG_NAME_AUTO)

SUFFIX = $(MACHINE)-$(CODENAME)-$(REVISION)$(REBUILD)-$(HOST)-$(CONFIG_NAME)

CPU ?= $(MACHINE)

NPROCESSORS := $(shell getconf _NPROCESSORS_ONLN)
NUM_CORES ?= $(NPROCESSORS)
PARALLEL ?= -j$(NPROCESSORS)

# Where intermediate files should go.  Ideally this should be an empty
# string which means 'system default', but dpkg-buildpackage doesn't
# handle this.
BUILD_TMP ?= /tmp

# Make sure we use time, not the built-in time
TIME = /usr/bin/time
PASSWORD_FILE = ~/.config/cbuild/password
# Where to publish the results.  Overridden by benchmarks.
PUBLISH_SUFFIX ?= build
RESULTS = results/$(PUBLISH_SUFFIX)/$(B)
# Where to push the results via rsync.  Set to 'local' to keep in a local directory.
PUBLISH_URL ?= cbuild@cbuild-master:/home/cbuild/public_html
BINARIES_URL ?= $(PUBLISH_URL)/binaries
GPGSIGN = gpg --no-use-agent --no-tty --batch -q --yes --armor --sign --detach-sig --passphrase-file $(PASSWORD_FILE)
CCAT = ccrypt -k $(PASSWORD_FILE) -c
RSYNC_PASSWORD = echo -n rsync | cat $(PASSWORD_FILE) - | sha1sum | cut -c-8
SCHEDULER_API ?= http://ex.seabright.co.nz/helpers/scheduler/api
# Notifies the scheduler that the slave has moved on to a step
PING_STEP ?= -wget --no-proxy -q -O - $(SCHEDULER_API)/update/$(HOST)/running/$(SUITE)-$(SNAME)$(1)

# Notification has been shifted into cbuildrc
# Who to notify, and how
NOTIFY ?= linaro-toolchain-builds@lists.launchpad.net
NOTIFY_ALWAYS ?= $(NOTIFY)
NOTIFY_FROM ?= Linaro Toolchain Builder <ec2-relay+cbuild@linaro.org>
SMTP_HOST ?= smtp.gmail.com:587
# echo -n password | ccrypt -e -K foo | base64 ...
SMTP_PASSWORD ?= O31wGeYCGTfz63niXLOQhYwbTP6/nm7wCyItun40FQHfZc9WD8xVpGG45kI=
SMTP_AUTH ?= -xu ec2-relay@linaro.org -xp `echo $(SMTP_PASSWORD) | base64 -d | $(CCAT)`

# Number of times to run each benchmark
BENCH_RUNS = 5

# Directory to build a suite into
LBUILD = $(B)/$(SUITE)

# Generic support for variants
-include $(B)/$(SUITE)-variants-$(VID).mk

# Name of the variant
VNAME ?=
ifeq ($(VNAME),)
# Variant specific build directory
VBUILD = $(LBUILD)/default
# Name to insert into a stamp name
SNAME = 
else
VBUILD = $(LBUILD)/$(VNAME)
SNAME = $(VNAME)-
endif

# Helper that records if a command failed
CHECK_FAILED = (touch $(@:%.stamp=%-failed.txt) && false)
