# Build and test crosstool-NG
#
SUITE = crosstool-ng
BUILD_OPTIONS = noarchive

-include local.mk
include $(TOPDIR)/lib/common.mk

include $(LIBDIR)/steps.mk

CONFIGURE_FLAGS =
SAMPLE ?= linaro-arm-linux-gnueabi

# Use a short name as the full path is more than bash can handle
C = $(TOP)/$(SUITE)
BUILDS = $(C)/builds

# Matches all build results
PATTERN = gcc*.{bz2,zip,xz,exe}

# Note that crosstool-NG gets confused if the top level make is called
# with -k.  Use -S to cancel out the -k
#
# Ensure the logs are copied out by logging if the build failed,
# copying, then failing if the log exists.

# Build.  Use ccache, as later stages build the same files.  Pull
# overrides from the job file.
build:
	cat /dev/null $(B).job | \
	grep ^CROSSTOOL_NG_ | \
	sed "s/^CROSSTOOL_NG_//" > $(C)/local.mk
	mkdir -p $(VBUILD)/ccache
	CCACHE_DIR=$(TOP)/$(VBUILD)/ccache \
	PATH=/usr/lib/ccache:$$PATH \
	$(MAKE) -C $(C) -S -f contrib/linaro/build.mk \
	> $(STEP).txt 2>&1 || touch $(STEP).err
	# Copy the logs out
	for i in $(BUILDS)/*; do \
		cp -a $$i/build.log $(STEP)-`basename $$i`.txt || true; \
	done
	-xz -f $(STEP)-*.txt
	# Fail if the earlier build failed
	[ ! -f $(STEP).err ]

testsuite:
	$(MAKE) -C $(C) -S -f contrib/linaro/build.mk check > $(STEP).txt 2>&1 || $(XCHECK_FAILED)

# Do nothing.  We've already installed
install:
	true

extract: $(B)/$(B).tar
	rm -rf $(C)
	mkdir -p $(C)
	tar xaf $< -C $(C) --strip-components=1
	# The i686 EC2 hosts are a bit sensitive.  Patch the config
	# files to not build too hard
	for i in $(C)/samples/linaro*/*.config; do \
		echo $$i; \
		sed -ir 's#CT_PARALLEL_JOBS=0#CT_PARALLEL_JOBS=$(NPROCESSORS)#' $$i; \
	done

get-sizes:
	echo du: >> $(STEP).txt
	-du -ab $(BUILDS)/*/$(PATTERN) >> $(STEP).txt

publish:
	-cp -a $(BUILDS)/*/$(PATTERN) $(RESULTS)
	make -f $(LIBDIR)/publish.mk SUITE=$(SUITE)

$(B)/$(B).tar:
	make -sf $(TOPDIR)/lib/fetch.mk fetch B=$(B)
