# Build and test EGLIBC using the compiler
#
SUITE = eglibc

-include local.mk
include $(TOPDIR)/lib/common.mk

# The testsuite sometimes locks up the tcpanda boards.  Disable for
# now.  See LP: #1097121.
BUILD_OPTIONS = nocheck

# As at eglibc_2.13-0ubuntu1, Ubuntu uses the following on ARM:
#  CC="gcc-4.5 -fno-stack-protector -U_FORTIFY_SOURCE"
#  CXX="g++-4.5 -fno-stack-protector -U_FORTIFY_SOURCE"
#  AUTOCONF=false
#  --host=arm-linux-gnueabi --build=arm-linux-gnueabi
#  --prefix=/usr
#  --without-cvs --enable-add-ons=libidn,"ports nptl "
#  --enable-profile
#  --without-selinux
#  --enable-stackguard-randomization
#  --with-pkgversion="Ubuntu EGLIBC 2.13-0ubuntu1" --with-bugurl="http://www.debian.org/Bugs/"
#  --with-headers=/scratch/michaelh/eglibc-2.13/debian/include
#  --enable-kernel=2.6.15


ifeq ($(MACHINE),armv7l)
TRIPLET = --host arm-linux-gnueabi --build arm-linux-gnueabi
endif

include $(LIBDIR)/steps.mk

configure:
	(cd $(VBUILD)/build && \
	CC="gcc -fno-stack-protector -U_FORTIFY_SOURCE" \
	CXX="g++ -fno-stack-protector -U_FORTIFY_SOURCE" \
	../../$(SUITE)-*/configure \
	--prefix=$(TOP)/$(VBUILD)/install \
	$(TRIPLET) \
	--without-cvs \
	--enable-add-ons=libidn,ports,nptl \
	--enable-profile \
	--without-selinux \
	--enable-stackguard-randomization \
	--enable-kernel=2.6.19 \
	$(WITHCPU) \
	) > $(STEP).txt 2>&1
	# The tests require the correct libgcc_s.so.1
	ln -sf `gcc -print-file-name=libgcc_s.so.1` $(VBUILD)/build
	ln -sf `gcc -print-file-name=libstdc++.so.6` $(VBUILD)/build

ifneq (,$(findstring $(SUITE),$(B)))
# Building eglibc itself
TARBALL = $(B)/$(B).tar
else
# Using eglibc as a testsuite
TARBALL = $(B)/$(SUITE)-2.15.tar
endif

extract-top: $(TARBALL)
	mkdir -p $(LBUILD)
	tar xaf $< -C $(LBUILD)

# Fetch, if needed
$(B)/%.tar:
	make -f $(TOPDIR)/lib/fetch.mk fetch F=$@
