# Fetches a binary from somewhere caching locally if needed
#

-include local.mk
include $(TOPDIR)/lib/common.mk

CACHE = cbuild@cbuild-cache:~/incoming/cache
ALT_CONFIG ?= None
XCONFIG ?= $(CONFIG_NAME)
SUITE ?= fetch-binary
INTO ?= $(VERSION)

THIS = $(LIBDIR)/fetch-binary.mk
# The binary we're looking for
PATTERN = $(VERSION)-$(MACHINE)-*-cbuild*$(XCONFIG).tar*z
#SILENT = -s

# Fetch, falling back to the alternate config if any.  The alternate
# config is used for the A8 to pick up an A9 toolchain if no A8
# version exists
fetch:
	$(call PING_STEP,$(VERSION))
	mkdir -p $(INTO)
	$(MAKE) $(SILENT) -f $(THIS) subfetch \
	|| $(MAKE) $(SILENT) -f $(THIS) subfetch XCONFIG=$(ALT_CONFIG)

# Fetch in priority order
subfetch:
	$(MAKE) $(SILENT) -f $(THIS) direct \
	|| $(MAKE) $(SILENT) -f $(THIS) s3 \
	|| $(MAKE) $(SILENT) -f $(THIS) cache \
	|| $(MAKE) $(SILENT) -f $(THIS) remote

# Get from the build store
direct:
	rsync $(RSYNCFLAGS) -ti "$(BINARIES_URL)/$(PATTERN)" $(INTO)
	echo Got direct

# Get from S3
s3:
	s3cmd get --force "$(BINARIES_URL)/$(PATTERN)" $(INTO)
	ls $(INTO)/$(PATTERN)
	echo Got from S3
	-rsync $(RSYNCFLAGS) -ti $(INTO)/$(PATTERN) $(CACHE)

# Get from the cache
cache:
	rsync $(RSYNCFLAGS) -ti "$(CACHE)/$(PATTERN)" $(INTO)
	echo Got from the cache

# Get from remote, then cache it
remote:
	rsync $(RSYNCFLAGS) --progress -ti -e "ssh -p 11000 -o BatchMode=yes" "$(VALIDATION)/$(VERSION)/$(PATTERN)" $(INTO)
	echo Got from remote
	-rsync $(RSYNCFLAGS) -ti $(INTO)/$(PATTERN) $(CACHE)
