# Fetches a snapshot from the standard location
#

-include local.mk
include $(TOPDIR)/lib/common.mk

F ?= $(B)/$(B).tar

CONTEXT ?= snapshots
AREA ?=
SNAPSHOTS ?= http://cbuild.validation.linaro.org/$(CONTEXT)

THIS = $(LIBDIR)/fetch.mk
#SILENT = -s

fetch: $(F)

%.tar:
#	$(MAKE) $(SILENT) -f $(THIS) $@-spawn 2>&1 | grep -v '^make\['
	$(MAKE) $(SILENT) -f $(THIS) $@-spawn 2>&1

%.tar.xz.cpt %.tar.bz2.cpt::
	$(MAKE) $(SILENT) -f $(THIS) $@-get-top 2>&1 | grep -v '^make\['

# Try the different file types
%-spawn:
	$(MAKE) -f $(THIS) $*-xdelta3 \
	|| $(MAKE) -f $(THIS) $*-direct-xz \
	|| $(MAKE) -f $(THIS) $*-direct-bz2

%-direct-bz2: %.bz2
	bunzip2 -f $<

%-direct-xz: %.xz
	unxz -f $<

%.xz %.bz2:
	$(MAKE) -f $(THIS) $@-get-top

%.tar-xdelta3: %.tar.xdelta3
	echo `xdelta3 printhdr $< | grep \(source\) | awk -F : '{ print $$2; }'` > $*.source
	$(MAKE) -f $(THIS) $(*D)/`cat $*.source`.xz-get-top AREA=/base
	unxz -f $(*D)/`cat $*.source`.xz
	xdelta3 -qfd $<
	-mv -f $(dir $*)/$(subst ^$(VARIANT),,$(notdir $*)).tar $*.tar
	rm -f $(*D)/`cat $*.source` $< $*.source

%.tar.xdelta3: %.tar.xdelta3.xz
	unxz -f $<

# Try the different layouts
%-get-top:
	$(MAKE) -f $(THIS) $*-get-local \
	|| $(MAKE) -f $(THIS) $*-get-https

# Fetch from a local tarballs directory
%-get-local:
	mkdir -p $(*D)
	path=$(TOPDIR)/tarballs/$(*F) && \
	echo "Trying $$path" && \
	cp -au $$path $*
	echo "Succeeded - got direct from the local store"

# Disabled support for sharded files
%-get-sharded:
	shard=`$(LIBDIR)/shard.sh $(shell basename $*)` && \
	$(MAKE) -f $(THIS) $*-get-https CONTEXT=$(CONTEXT)/$$shard

# Try the different servers
%-get-https:
	$(MAKE) -f $(THIS) $*-get-http

%-get-http:
	mkdir -p $(*D)
	url=$(SNAPSHOTS)$(AREA)/$(subst ^$(VARIANT),,$(*F)) && \
	echo "Trying $$url" && \
	$(TIME) -o $(*D)/fetch.tmp wget -q -O $* $$url
	$(MAKE) -f $(THIS) $*-test
	md5sum $* | tee -a $(@D)/md5sums.txt
	echo "Succeeded - took `grep elapsed $(*D)/fetch.tmp`"

# Can't test
%.tar.xz.cpt-test %.tar.bz2.cpt-test:
	true

# Test that the file was downloaded correctly
%.xz-test:
	unxz -t $*.xz

%.bz2-test:
	bunzip2 -t $*.bz2
