# Build GCC
#
SUITE = gcc

-include local.mk
include $(TOPDIR)/lib/common.mk

LANGUAGES ?= c,c++,objc,obj-c++,fortran

ifndef CROSS_COMPILING
# libgomp lacks the return code wrapper so all execute tests fail
# libmudflap causes QEMU to core dump on mudflap violation
CHECK_EXTRAS += \
	target-libgomp target-libmudflap
endif

CHECK ?= \
	gcc \
	intl libcpp libdecnumber libiberty lto-plugin \
	target-libgcc \
	target-libquadmath target-libssp \
	target-libgfortran \
	target-libstdc++-v3 \
	target-libffi \
	$(CHECK_EXTRAS)

# 4.6 has the following target tests:
#  boehm-gc examples gperf libada libffi libgcc libgfortran libgloss
#  libgo libjava libmudflap libobjc libquadmath libssp libstdc++-v3
#  libtermcap newlib qthreads rda winsup zlib
#
# In a cross build, the following are present:
#  libgcc libgfortran libgomp libmudflap libquadmath libssp libstdc++-v3
#
# Add libffi

BOOTSTRAP ?=
ARCHIVE_BASE = $(B)
PREBUILT = $(B)/prebuilt
NEWLIB_VERSION ?= newlib-1.21~20120927+gitb44fba4

# Pass variables down to steps.mk
BUILDTARGET = $(GCCBUILDTARGET)

# Add in LTO for non-4.4 builds
ifeq (,$(findstring 4.4,$(B)))
# Note that we have to use this form.  A += adds a space.
LANGUAGES := $(LANGUAGES),lto
endif

CONFIGURE_FLAGS = \
	--prefix=$(TOP)/$(VBUILD)/install \
	--enable-languages=$(LANGUAGES) \
	--enable-linker-build-id \
	$(BOOTSTRAP) \
	$(GCCCONFARGS) \
	$(GCC_CONFIGURE_EXTRAS)

PATH := $(TOP)/$(PREBUILT)/bin:$(TOP)/$(TOPDIR)/etc/$(TRIPLET)/$(SUITE):$(PATH)
export PATH

include $(LIBDIR)/steps.mk

testsuite:
	# QEMU user mode can't handle the split prebuilt/GCC runtime.  Merge in a hacky
	# way
	[ -z "$(QEMU_LD_PREFIX)" ] || cp -fua $(VBUILD)/install/*-*-*/lib/* $(SYSROOT)/usr/lib
	# When cross-testing, a few libraries need to be accessible next to ld-linux.so
	[ -z "$(QEMU_LD_PREFIX)" ] ||  ln -s $(TRIPLET)/libdl.so.2 $(TRIPLET)/libm.so.6 $(TRIPLET)/libpthread.so.0 $(SYSROOT)/lib/
	for i in $(CHECK); do \
		echo "Running check for $$i"; \
		echo >> $(STEP).txt; \
		echo $$i:: >> $(STEP).txt; \
		DEJAGNU=$(GCC_DEJAGNU) \
		make -k $(CHECK_PARALLEL) -sC $(VBUILD)/build maybe-check-$$i >> $(STEP).txt 2>&1; \
		date --rfc-3339=seconds >> $(STEP).txt; \
	done
	(cd $(VBUILD)/build && ../../gcc-*4.*/contrib/test_summary) > $(STEP)-summary.txt

configure:
	echo $(CONFIGURE_FLAGS) > $(STEP).txt
	gcc --version >> $(STEP).txt 2>&1
	(cd $(VBUILD)/build && ../../gcc-*4.*/configure $(CONFIGURE_FLAGS)) >> $(STEP).txt 2>&1

build:
#	make -k $(PARALLEL) -C $(VBUILD)/build $(BUILDTARGET) $(GCC_BUILD_FLAGS) > $(STEP).txt 2>&1
	make -k $(PARALLEL) -C $(VBUILD)/build $(BUILDTARGET) $(GCC_BUILD_FLAGS) 2>&1 | tee $(STEP).txt

post-archive:
	make -f $(LIBDIR)/publish.mk SUITE=$(B) binary-only

extract-top: $(B)/$(B).tar $(B)/$(NEWLIB_VERSION).tar $(EXTRACT_TOP_EXTRAS)
	tar xaf $< -C $(LBUILD)
	rm -f $<
	# Hack: work around a temporary aarch64 problem
	cat /dev/null $(wildcard $(TOPDIR)/files/gcc/*$(CONFIG)*patch) | (cd $(LBUILD)/gcc-* && patch -p1)
	# Pull in newlib so it's there if needed
	tar xaf $(B)/$(NEWLIB_VERSION).tar -C $(LBUILD)
	# Hack: add aarch64 newlib work-arounds
	cat /dev/null $(wildcard $(TOPDIR)/files/newlib/*$(CONFIG)*patch) | (cd $(LBUILD)/newlib-* && patch -p1)
	ln -f -s $(TOP)/$(LBUILD)/newlib-*/newlib $(LBUILD)/gcc-*
	ln -f -s $(TOP)/$(LBUILD)/newlib-*/libgloss $(LBUILD)/gcc-*

extract-prebuilt: $(B)/$(PREBUILT_TOOLS)
	rm -rf $(PREBUILT) && mkdir -p $(PREBUILT)
	tar xaf $< -C $(PREBUILT) --strip-components=1
	# Hack: copy crti and friends to a multiarch-ish path
	mkdir -p $(PREBUILT)/$(TRIPLET)/libc/usr/lib/$(TRIPLET)
	-cp -u $(PREBUILT)/$(TRIPLET)/libc/usr/lib/crt* $(PREBUILT)/$(TRIPLET)/libc/usr/lib/$(TRIPLET)

# Fixup the difference between the hard float and GCC's triplet
fixup-triplet:
	cd $(PREBUILT) && ln -s $(TRIPLET) $(ALT_TRIPLET)
	cd $(PREBUILT)/bin && \
	for i in $(TRIPLET)*; do \
	ln -s $$i $$(echo $$i | sed s/$(TRIPLET)/$(ALT_TRIPLET)/); \
	done

# Fixup multiarch for non-multiarch compilers
fixup-multiarch:
	cd $(PREBUILT)/$(TRIPLET)/libc/usr/include && \
	for i in asm gnu bits sys; do \
	if [ -d $(TRIPLET)/$$i ]; then rm -rf $$i; ln -sf $(TRIPLET)/$$i; fi \
	done
	cd $(PREBUILT)/$(TRIPLET)/libc/usr/lib && \
	ls $(TRIPLET)/*crt*.o | xargs -L 1 ln -sf

# Mildly hacky.  Put the Foundation model simulator in the prebuilt path
extract-emulator: $(B)/$(PREBUILT_EMULATOR)
	mkdir -p $(PREBUILT)/bin
	$(CCAT) $< | unxz | tar xf - -C $(PREBUILT)/bin --strip-components=1

# Fetch, if needed
$(B)/%.tar:
	make -f $(TOPDIR)/lib/fetch.mk fetch F=$@

$(B)/%.tar.xz.cpt:
	make -f $(TOPDIR)/lib/fetch.mk fetch F=$@ CONTEXT=snapshots/prebuilt
