# Build libav as a benchmark and NEON build test
#
SUITE = libavbench
PUBLISH_SUFFIX = benchmarks
BUILD_OPTIONS = nocheck noarchive

BUILDFLAGS = V=1

include local.mk
include $(TOPDIR)/lib/common.mk
include $(LIBDIR)/steps.mk

DATA = $(B)/libavbench-data-r1.tar

# We used to disable everything but there's use in being a compile
# test as well.
CONFIGURE_FLAGS += --disable-pthreads

# See --disable-asm, --enable-small, --extra-cflags=-mfpu=neon

VCFLAGS ?= -O3
DISABLE_ASM ?=  --disable-asm
REMOVE_CFLAGS ?= -fno-tree-vectorize

CONFIGURE_FLAGS += --optflags="$(VCFLAGS)"
CONFIGURE_FLAGS += $(DISABLE_ASM)

run:
	-for f in $(wildcard $(LBUILD)/libavbench-data*/*); do \
	echo >> $(STEP).txt; \
	echo `basename $$f`:: >> $(STEP).txt; \
	cat $$f > /dev/null; \
	for i in $$(seq $(BENCH_RUNS)); do \
	$(TIME) -o $(STEP).txt -a $(VBUILD)/install/bin/ffmpeg -benchmark -loglevel quiet -i $$f -f null -y /dev/null > $(VBUILD)/benchmark.tmp; \
	cat $(VBUILD)/benchmark.tmp >> $(STEP).txt; \
	done; done

configure:
	echo $(CONFIGURE_FLAGS) >> $(STEP).txt
	(cd $(VBUILD)/build && \
	../../libav-0*/configure --prefix=$(TOP)/$(VBUILD)/install $(CONFIGURE_FLAGS) \
	) >> $(STEP).txt 2>&1
	# Strip out any unwanted default flags
	for i in $(REMOVE_CFLAGS); do sed -i~ s#$$i## $(VBUILD)/build/config.mak; done
	grep FLAGS $(VBUILD)/build/config.mak >> $(STEP).txt
	touch $@

extract-top: $(TOPDIR)/files/libav-*.xz $(DATA)
	tar xaf $< -C $(LBUILD)
	tar xaf $(DATA) -C $(LBUILD)
	rm -f $(DATA)

$(DATA):
	make -f $(TOPDIR)/lib/fetch.mk fetch F=$@ CONTEXT=misc/cbuild
