"""Take a file with blank lines between sections, check the shell
style glob predicate, and spit out one file per section.
"""
import sys
import fileinput
import os
import re
import platform
import fnmatch

def main():
    filter = sys.argv[1].split()
    args = sys.argv[2:]

    sections = []
    section = []

    for line in fileinput.input(args):
        line = line.rstrip()

        if not line:
            if section:
                sections.append(section)
            section = []
        elif line == '.':
            section.append('')
        else:
            section.append(line)

    if section:
        sections.append(section)

    root = os.path.basename(fileinput.filename())
    base, ext = os.path.splitext(root)

    for i, section in enumerate(sections):
        include = True

        # See if the section has a condition on it
        if section:
            match = re.match('\[ (\S.+)\s+\]', section[0])

            if match:
                include = eval(match.group(1))
                section = section[1:]

        if include and filter:
            for line in section:
                match = re.match('VNAME\s*=\s*(\S+)', line)

                if match:
                    if any(fnmatch.fnmatch(match.group(1), x) for x in filter):
                        break
            else:
                include = False

        # See if the section matches the filter
        if include:
            with open('%s-%d%s' % (base, i+1, ext), 'w') as f:
                f.write('\n'.join(section) + '\n')

if __name__ == '__main__':
    main()
