#!/bin/bash
# yep, this one is fragile!

# exit immediately if anything goes wrong
set -e

if [ $# -ne 2 ]; then
  echo echo >&2 "usage: $0 /path/to/oe-core-qemu-image.ext3"
  exit 1
fi

TARGET="$1"
IMAGE="$2"
IMAGE_DIR=$(dirname $IMAGE)
QEMU_DIR="${IMAGE_DIR%%deploy*}sysroots/$(uname -m)-linux/usr/bin"
COMMON_QEMU_OPTS="-serial pipe:${IMAGE_DIR}/qmpipe -display none -snapshot -no-reboot"
TIMEOUT=180

# ensure we remove the named pipes
trap atexit EXIT
atexit(){
  rm -f ${IMAGE_DIR}/qmpipe.in ${IMAGE_DIR}/qmpipe.out
}

run_qemu ()
{
  local QEMU_CMD="$1"
  local QEMU_LOG="$2"

  # create named pipes to communicate with the qemu guest"
  mkfifo ${IMAGE_DIR}/qmpipe.in ${IMAGE_DIR}/qmpipe.out

  # reading from qmpipe.out will block until the qemu dies
  #tee ${QEMU_LOG} < ${IMAGE_DIR}/qmpipe.out &
  cat ${IMAGE_DIR}/qmpipe.out > ${QEMU_LOG} &

  echo "start qemu in background"
  eval "${QEMU_CMD} &"
  local QEMU_PID="$!"

  echo "wait till the system has booted"
  local i
  for i in `seq ${TIMEOUT}`; do
    if grep -q --binary-files=text 'login\:' ${QEMU_LOG}; then
      break
    fi
    sleep 1
  done
 
  echo "login as root"
  echo "root" > ${IMAGE_DIR}/qmpipe.in
  sleep 1

  echo "issue some command"
  echo "free" > ${IMAGE_DIR}/qmpipe.in
  sleep 1

  echo "kill qemu"
  kill -9 ${QEMU_PID}

  echo "deleting the named pipes"
  rm ${IMAGE_DIR}/qmpipe.in ${IMAGE_DIR}/qmpipe.out

  grep -A3 --binary-files=text "total         used         free       shared      buffers" ${QEMU_LOG} 
}


case "${TARGET}" in
  qemuarmv7a)
    QEMU="${QEMU_DIR}/qemu-system-arm \
-M vexpress-a9 -m 1024 \
-kernel ${IMAGE_DIR}/zImage-qemuarmv7a.bin \
-drive file=${IMAGE},if=sd,cache=writeback \
--append \"rw console=ttyAMA0,38400n8 console=tty root=/dev/mmcblk0\""
    ;;
  qemuarm)
    QEMU="${QEMU_DIR}/qemu-system-arm \
-M versatilepb -m 128 \
-usb -usbdevice wacom-tablet \
-kernel ${IMAGE_DIR}/zImage-qemuarm.bin \
-hda ${IMAGE} \
--append \"rw console=ttyAMA0,115200 console=tty root=/dev/sda highres=off\""
    ;;
  qemumips)
    QEMU="${QEMU_DIR}/qemu-system-mips \
-M malta -m 128 \
-vga cirrus -usb -usbdevice wacom-tablet \
-kernel ${IMAGE_DIR}/vmlinux-qemumips.bin \
-hda ${IMAGE} \
--append \"rw root=/dev/hda console=ttyS0 console=tty\""
    ;;
  qemuppc)
    QEMU="${QEMU_DIR}/qemu-system-ppc \
-M mac99 -cpu G4 -m 128 \
-usb -usbdevice wacom-tablet  \
-kernel ${IMAGE_DIR}/vmlinux-qemuppc.bin \
-hda ${IMAGE} \
--append \"rw root=/dev/hda console=ttyS0 console=tty\""
    ;;
  qemux86)
    QEMU="${QEMU_DIR}/qemu \
-m 128 \
-vga vmware -enable-gl -usb -usbdevice wacom-tablet \
-kernel ${IMAGE_DIR}/bzImage-qemux86.bin \
-hda ${IMAGE} \
--append \"rw vga=0 root=/dev/hda oprofile.timer=1\""
    ;;
  qemux86-64)
    QEMU="${QEMU_DIR}/qemu-system-x86_64 \
-m 128 \
-vga vmware -enable-gl -usb -usbdevice wacom-tablet \
-kernel ${IMAGE_DIR}/bzImage-qemux86-64.bin \
-hda ${IMAGE} \
--append \"rw vga=0 root=/dev/hda oprofile.timer=1\""
    ;;
  *)
    echo echo >&2 "unknown target: ${TARGET}"
    exit 1
    ;;
esac

echo "boot $(basename ${IMAGE}) using QEMU"
run_qemu "${QEMU} ${COMMON_QEMU_OPTS}" "${IMAGE%.ext3}_console.log"

