# Runs all variants of a build
#
-include local.mk
include $(TOPDIR)/lib/common.mk

# The suite to run.  Matches a lib/?.mk file and a lib/?-variants.mk
AROUND ?= error
# Where to pull variants from if there's no suite specific version
VARIANT_SRC ?= default
# Which variants in the src to run.  Uses shell wildcards
VARIANT_LIST ?= *

# Stage 1.  Make the files for each variant
$(B)/$(AROUND)-variants.stamp: $(B)/$(AROUND)-mkvariants.stamp
	$(MAKE) -f $(LIBDIR)/runvariants.mk run || (touch $(B)/$(AROUND)-failed.txt && false)

# Generate a file for each variant
$(B)/$(AROUND)-mkvariants.stamp: $(B)/$(AROUND)-variants.mk
	mkdir -p $(@D)
	python $(LIBDIR)/mkvariants.py "$(VARIANT_LIST)" $<
	mv $(AROUND)-variants*.mk $(B)
	touch $@

# Pull in either the per-product or default variants
$(B)/$(AROUND)-variants.mk: $(wildcard $(LIBDIR)/$(AROUND)-variants.mk) $(LIBDIR)/$(VARIANT_SRC)-variants.mk
	cp $< $@

# Stage 2.  All of the variants have been made.  Run each one.
VARIANTS = $(wildcard $(B)/$(AROUND)-variants-*.mk)

run: $(VARIANTS:%.mk=%.stamp)

$(B)/$(AROUND)-variants-%.stamp:
	$(MAKE) -f $(LIBDIR)/$(AROUND).mk VID=$*
