# Builds and runs the Ski benchmark
#
SUITE = skiabench
PUBLISH_SUFFIX = benchmarks
BUILD_OPTIONS = noarchive nocheck

include local.mk
include $(TOPDIR)/lib/common.mk
include $(LIBDIR)/steps.mk

VCFLAGS ?= -O2 $(TUNE_FLAGS)
VCFLAGS += -fstrict-aliasing

UP = ../../..

RUN_FLAGS = -rotate -scale -clip -config 8888

# Number of iterations for each sub-benchmark.  Calibrated so that
# each run takes about 10 s on a Cortex-A9
fps_bitmap_565_scale = 200
fps_bitmap_565_noscale = 200
fps_bitmap_X888_scale = 340
fps_bitmap_X888_noscale = 300
fps_bitmap_8888_scale = 200
fps_bitmap_8888_noscale = 180
fps_blend = 700
fps_fill = 4800
repeatTile_index8 = 14
repeatTile_4444 = 12
repeatTile_565 = 14
repeatTile_8888 = 14
bitmap_index8 = 6
bitmap_index8_A = 10
bitmap_4444 = 6
bitmap_4444_A = 10
bitmap_565 = 10
bitmap_8888 = 6
bitmap_8888_A = 10
polygon = 20
lines = 80
points = 240
rrects3 = 140
rrects1 = 40
ovals3 = 160
ovals1 = 50
rects3 = 80
rects1 = 20

# Names of all of the sub-benchmarks
ALL_NAMES = \
	fps_bitmap_565_scale fps_bitmap_565_noscale fps_bitmap_X888_scale \
	fps_bitmap_X888_noscale fps_bitmap_8888_scale fps_bitmap_8888_noscale \
	fps_blend fps_fill repeatTile_index8 \
	repeatTile_4444 repeatTile_565 repeatTile_8888 \
	bitmap_index8 bitmap_index8_A bitmap_4444 \
	bitmap_4444_A bitmap_565 bitmap_8888 \
	bitmap_8888_A polygon lines \
	points rrects3 rrects1 \
	ovals3 ovals1 rects3 \
	rects1

extract: $(TOPDIR)/files/skia-*.xz
	tar xaf $< -C $(VBUILD)

install:
	ln -s $(PWD)/$(VBUILD)/skia-*/out $(VBUILD)/install

build:
	make -s $(PARALLEL) -C $(VBUILD)/skia-* CFLAGS="$(VCFLAGS)" CFLAGS_SSE2= bench > $(STEP).txt 2>&1

run:
	for i in $$(seq $(BENCH_RUNS)); do \
		echo Run $$i:: >> $(STEP).txt; \
		$(TIME) -o $(STEP).time -a $(MAKE) -sf $(CHILD) run-all >> $(STEP).txt; \
	done

run-all: $(ALL_NAMES:%=run-one-%)

run-one-%:
	$(VBUILD)/skia-*/out/bench/bench $(RUN_FLAGS) -repeat $(value $*) -match $*
