# Build and run the SPEC CPU2000 benchmarks
#
SUITE = spec2000
PUBLISH_SUFFIX = benchmarks
BUILD_OPTIONS = nocheck noarchive

-include local.mk
include $(TOPDIR)/lib/common.mk
include $(LIBDIR)/steps.mk

# The configurations are for 32 bit only.  Hack around this on x86_64
# to build in 32 bit mode.  This makes the results invalid but still
# lets us use SPEC as a testsuite
ifeq ($(MACHINE),x86_64)
TUNE_FLAGS += -m32
endif

VCFLAGS ?= -O2 $(TUNE_FLAGS)
# Always use -fno-common
VCFLAGS += -fno-common
TESTS ?= int fp

SPECCONFIG ?= linux-armv7l-gcc4-linaro-cbuild
BASE_EXTENSION ?= cbuild-$(notdir $(VBUILD))

ifdef WITH_PGO
EXTENSION ?= $(BASE_EXTENSION)-pgo
else
EXTENSION ?= $(BASE_EXTENSION)
endif

export EXTENSION

# SPEC relocate uses the path in a regex and can't handle characters
# like '+'.  Use a symlink for something short and clean
NICEBUILD = $(SUITE)

run:
	make -C $(VBUILD)/cpu2000* run CONFIG=$(SPECCONFIG) WORKLOAD=ref TESTS="$(TESTS)"
	for i in $(VBUILD)/cpu2000*/result/C*.{asc,raw}; do \
	echo >> $(STEP).txt; \
	echo $$i:: >> $(STEP).txt; \
	cat $$i >> $(STEP).txt; \
	done
	# Check to see if any errors happened in the run
	if cat $(STEP).txt | grep -v reportable | grep errors > $(STEP).tmp; then \
	mv $(STEP).tmp $(STEP)-failed.txt; fi

build:
	echo VCFLAGS=$(VCFLAGS) >> $(STEP).txt 2>&1
	echo $$EXTENSION
	make -C $(NICEBUILD) build CONFIG=$(SPECCONFIG) TESTS="$(TESTS)" >> $(STEP).txt 2>&1

extract:
	cp -alf $(LBUILD)/cpu2000* $(VBUILD)
	ln -sf $(VBUILD)/cpu2000* $(NICEBUILD)

configure:
	# Create the config file
	(cd $(NICEBUILD) && make relocate set-cbuild EXTENSION=$(BASE_EXTENSION) CONFIG=$(SPECCONFIG:%-cbuild=%) OPTIMISE="$(VCFLAGS)") >> $(STEP).txt 2>&1

extract-top: $(TOPDIR)/files/cpu2000-1*.tar*cpt
	# Extract SPEC
	$(CCAT) $< | tar xJf - -C $(LBUILD)
	chmod -R +w $(LBUILD)
	# and the tools for this architecture
	$(CCAT) $(TOPDIR)/files/cpu2000tools-*$(MACHINE)$(FLOAT_SUFFIX).tar*cpt | tar xJf - -C $(LBUILD)/cpu2000*
	# and the helper scripts
	tar xaf $(TOPDIR)/files/spec2000-linaro*.tar* -C $(LBUILD)/cpu2000* --strip-components=1

archive:
	true

install:
	rm -rf $(VBUILD)/install
	mkdir $(VBUILD)/install
	cd $(VBUILD) && for i in cpu2000*/benchspec/C*/*; do \
	if [ -d $$i/exe ]; then cp -a $$i/exe/* install/`basename $$i`; fi; done
