# Use Ubuntu as a testsuite
#
SUITE = ubutest
BUILD_OPTIONS = nocheck nobench noarchive

-include local.mk
include $(TOPDIR)/lib/common.mk

include $(LIBDIR)/steps.mk

DIST = precise

HAS_CHECK = \
	python2.7 \
	eglibc \
	base-files \
	base-passwd \
	bash \
	coreutils \
	dash \
	debianutils \
	diffutils \
	dpkg \
	e2fsprogs \
	findutils \
	grep \
	gzip \
	hostname \
	ncurses \
	perl \
	python-defaults \
	sed \
	shadow \
	tar \
	util-linux

ALL = $(HAS_CHECK)

# Don't check or benchmark as part of the main build
DEB_BUILD_OPTIONS += nocheck nobench

# Most of the packages ignore parallel= so we build the packages in
# parallel instead.  michaelh tried passing -j2 to dpkg-buildpackage
# but most steps in the build are serial, and some packages had
# phantom install errors.
build:
	$(MAKE) $(PARALLEL) -f $(LIBDIR)/$(SUITE).mk build-many

# Child rule that pulls in all of the packages
build-many: $(ALL:%=$(STEP)-%-build.stamp)

# Build a single package.  Always toast and extract as otherwise d-bp
# will undo the previous build before continuing.
#
# NJOBS is a hack.  eglibc is terribly slow to build but pulls the
# parallel level from NJOBS.  Python also does this but nothing else
# does, so it's safe to set NJOBS from the environment for all
# packages.
#
# -a is a hack.  debuild queries GCC for the host architecture, sees
# arm-linux-gnueabi on a hard float system, and assumes cross
# compiling.  Force host == build.
#
$(STEP)-%-build.stamp:
	# Start afresh
	rm -rf $(VBUILD)/$**
	cd $(VBUILD) && dpkg-source -x ../sources/$**.dsc
	# Remove any leftover tarballs
	-rm -f $(VBUILD)/*$**
	# Apply any patches
	cat /dev/null $(TOPDIR)/files/$(SUITE)/$(DIST)/$*-*.patch | patch -p1 -d $(VBUILD)/$**
	# Build
	(cd $(VBUILD)/$** && \
	NJOBS=$(CHECK_PARALLEL) DEB_BUILD_OPTIONS="$(DEB_BUILD_OPTIONS)" dpkg-buildpackage -uc -us -b -d -Tbuild -a$(shell dpkg-architecture -qDEB_BUILD_ARCH) \
	) > $(STEP)-$*.txt 2>&1 || touch $(STEP)-$*-failed.txt
	touch $@

# Fetch all of the source packages in one step
extract-top:
	mkdir -p $(LBUILD)/sources
	cd $(LBUILD)/sources && wget -nv -N -nd -r -np http://cbuild.validation.linaro.org/source/$(DIST)/

install:
	true

# Install all the build dependencies for this suite.  Called manually.
build-dep:
	sudo apt-get build-dep $(ALL)
	sudo apt-get install time python-irclib wget gnupg ccrypt rsync coreutils sendemail bzr xdelta3 xz-utils unzip openssh-client

.PRECIOUS:: $(STEP)-%-build.stamp
