# Build the V8 JavaScript engine and run the SunSpider benchmark

SUITE = v8
PUBLISH_SUFFIX = benchmarks
BUILD_OPTIONS = noarchive nocheck

include local.mk
include $(TOPDIR)/lib/common.mk
include $(LIBDIR)/steps.mk


# Map the current architecture to v8 build targets
ifeq ($(MACHINE), i686)
  TARGET = ia32.release
else ifeq ($(MACHINE), armv7l)
  TARGET = arm.release
else ifeq ($(MACHINE), x86_64)
  TARGET = x64.release
else
  TARGET = native.release
endif

# The V8 make flags
V8_MAKEFLAGS = \
	$(TARGET) \
	library=shared

# Compiler flags - be a little less strict to avoid build errors 
V8_CXXFLAGS += -Wno-array-bounds -Wno-strict-overflow

JS_SHELL = $(PWD)/$(VBUILD)/install/shell

run:
	cd $(LBUILD)/SunSpider*; \
	./sunspider --shell=$(JS_SHELL) --runs 50 --args=--expose_gc >> $(TOP)/$(STEP).txt

build:
	make $(PARALLEL) -C $(VBUILD)/$(SUITE)* $(V8_MAKEFLAGS) CXXFLAGS="$(VCFLAGS) $(V8_CXXFLAGS)" >> $(STEP).txt 2>&1

install:
	ln -s $(TOP)/$(VBUILD)/$(SUITE)*/out/*.release $(VBUILD)/install

extract: $(TOPDIR)/files/v8-*.xz $(TOPDIR)/files/gyp-*.xz
	tar xaf $< -C $(VBUILD)
	tar xaf $(TOPDIR)/files/gyp-*.xz -C $(VBUILD)/v8-*/build

extract-top: $(TOPDIR)/files/SunSpider*.xz
	tar xaf $< -C $(LBUILD)
