# Build and run libvorbis as a benchmark
#
SUITE = vorbisbench
PUBLISH_SUFFIX = benchmarks

include local.mk
include $(TOPDIR)/lib/common.mk
include $(LIBDIR)/steps.mk

INTO := $(shell mktemp --tmpdir -d cbuild-vorbisbench.XXXXXXXX)

CONFIGURE_FLAGS =

VCFLAGS ?= -O2 $(TUNE_FLAGS)
VCFLAGS += -Wall -Wextra -D_REENTRANT -fsigned-char -Wdeclaration-after-statement  -DUSE_MEMORY_H

# PENDING: Vorbis uses -ffast-math by default

run:
	for f in $(TOPDIR)/files/vorbis/*.ogg; do \
	echo $$f >> $(STEP).txt; \
	for i in $$(seq $(BENCH_RUNS)); do \
	$(TIME) -o $(STEP).txt -a $(VBUILD)/build/examples/decoder_example < $$f > $(INTO)/`basename $$f .ogg`.wav; \
	done; \
	for i in $$(seq $(BENCH_RUNS)); do \
	$(TIME) -o $(STEP).txt -a $(VBUILD)/build/examples/encoder_example < $(INTO)/`basename $$f .ogg`.wav > $(INTO)/`basename $$f`; \
	done; \
	rm -f $(INTO)/*; \
	done
	rm -rf $(INTO)
	touch $@

build:
	$(TIME) -o $(STEP).time make -k $(PARALLEL) -C $(VBUILD)/build CFLAGS="$(VCFLAGS)" > $(STEP).txt 2>&1

configure:
	echo CFLAGS=$(VCFLAGS) > $(STEP).txt
	(cd $(VBUILD)/build && ../../libvorbis-*/configure --prefix=$(TOP)/$(VBUILD)/install $(CONFIGURE_FLAGS)) >> $(STEP).txt 2>&1

extract-top: $(TOPDIR)/files/libvorbis-*
	tar xaf $< -C $(LBUILD)
