#!/bin/bash

# run.sh is special and needs to be able to run from checkout.
. $(dirname $0)/siterc
sync=${CBUILD_SYNC:-cbuild@cbuild-master:/home/cbuild/lib/cbuild}
api=${SCHEDULER_API:-http://cbuild.validation.linaro.org/helpers/scheduler/api}

wget="wget --no-proxy -q"
rsyncflags="-zlrt --exclude=archive --timeout=60"

# Lock down the locale
export LANG=C

hostname=`cat /etc/hostname`

# Send some basic statistics
./report.sh updating

set -e

# Nuke any past builds or jobs
rm -rf slaves
# Pull the main body across, deleting any extras
rsync $rsyncflags -C --del --exclude=$hostname "$sync/*" .
# Pull .bzr across for the version
rsync $rsyncflags "$sync/.bzr" .
# Copy the host specific files across
rsync $rsyncflags -C "$sync/slaves/$hostname" slaves || true

# See if we need to reboot
./lib/rebootif.sh $api $hostname

# Grab the next job, if any
tmpf=`mktemp`
$wget -O $tmpf $api/next/$hostname
# First line is the job name
next=`head -n 1 $tmpf`
# Remaining lines are options
sed -n '2,$ p' $tmpf > slaves/$hostname/`basename $next`
rm -f $tmpf

# Enable core files for the GDB testsuite
ulimit -c unlimited

# Notify and go!
$wget -O - $api/update/$hostname/running

make -k -C slaves/$hostname -f build.mk 2>&1 | tee slaves/$hostname/toplevel.txt

$wget -O - $api/update/$hostname/idle
