# Slave build script
#
# Scans all of the tarballs in this directory and makes sure each is built
#
-include local.mk
include $(TOPDIR)/lib/common.mk

TARBALLS := $(wildcard $(LIMIT)*.job)
REVISIONS := $(TARBALLS:%.job=%)
PWD := $(shell pwd)
BUILD_ALL_ ?= all

ALL_BENCHMARKS = bzip2bench coremark libavbench gmpbench gnugo skiabench vorbisbench benchnt skiabench denbench eembc eembc_office spec2000
# Subset to run by default
BENCHMARKS ?= coremark eembc eembc_office denbench

all: $(REVISIONS:%=final/%.stamp)

publish: $(REVISIONS:%=%/publish.stamp)
	true

# Stage 1: log the start, spawn the file again, trap any failures, and always publish
final/%.stamp: %/start.stamp %/spawn.stamp %/publish.stamp %/notify.stamp
	mkdir -p $(@D)
	touch $@

final/ubutest-%.stamp:
	$(MAKE) -f $(LIBDIR)/build.mk final/$*.stamp BUILD_ALL_=ubutest

final/test-oecore-%.stamp:
	$(MAKE) -f $(LIBDIR)/build.mk final/$*.stamp BUILD_ALL_=oecore

# Special rule for benchamrks that hands straight off to the GCC run
final/benchmarks-gcc-%.stamp:
	$(MAKE) -f $(LIBDIR)/build.mk final/gcc-$*.stamp BUILD_ALL_=benchmarks BTOP=benchmarks-gcc-$* PUBLISH_SUFFIX=benchmarks

# Run just SPEC 2000
final/benchmarks-spec2000-%.stamp:
	$(MAKE) -f $(LIBDIR)/build.mk final/$*.stamp BUILD_ALL_=benchmarks BTOP=benchmarks-spec2000-$* PUBLISH_SUFFIX=benchmarks BENCHMARKS=spec2000

final/porter.stamp:
	wget --no-proxy -q -O - $(SCHEDULER_API)/update/$(HOST)/porter/ready
	sleep 48h

%/start.stamp: %/host.txt
	mkdir -p $(@D)
	-wget --no-proxy -q -O - $(SCHEDULER_API)/update/$(HOST)/start/$*
	-grep ^JOB_ID $(@D).job >> $(@D)/started.txt
	touch $@

%/spawn.stamp:
	$(MAKE) -f $(LIBDIR)/build.mk $(@D)/$(BUILD_ALL_).stamp || touch $(@D)/failed.txt
	date --rfc-3339=seconds -u > $(@D)/finished.txt
	touch $@

%/publish.stamp:
	$(MAKE) -f $(LIBDIR)/publish.mk B=$(@D)
	touch $@

# Touch straight away so that the email doesn't go out multiple times.
# Build the body with the correct $B so that configs different from
# the host config get picked up
%/notify.stamp:
	mkdir -p $(@D)
	touch $@
ifeq ($(NONOTIFY),)
	$(MAKE) -f $(LIBDIR)/build.mk $*/notify.body B=$(@D)
	mkdir -p $(@D)
	echo "[cbuild] $${BTOP:-$(@D)} $(CONFIG_NAME) completed" > $@.subject
	# Change the subject if anything failed
	[ -z "$$(ls $(@D)/*failed.txt)" ] || echo "[cbuild] Failed: $${BTOP:-$(@D)} $(CONFIG_NAME) failed" > $@.subject
	cat $*/notify.body | sendemail -q -f "$(NOTIFY_FROM)" -t "$(NOTIFY_ALWAYS)" -u "`cat $@.subject`" -s $(SMTP_HOST) $(SMTP_AUTH)
endif

%/notify.body:
	mkdir -p $(@D)
	# Summarise the DejaGnu results, if any
	summary=`cat /dev/null $(@D)/*-testsuite.txt | grep -E "^[A-Z]+:" | sed -r 's/^([A-Z]+):.+/\1/' | sort | uniq -c`; \
	summary=`echo $$summary`; \
	job=$(subst ^,%5E,$(@D)); \
	topjob=$${BTOP:-$$job}; \
	cat $(LIBDIR)/notify.txt \
	| grep -v ^# \
	| sed s/@HOST@/$(HOST)/ \
	| sed s/@CONFIG_NAME@/$(CONFIG_NAME)/ \
	| sed "s/@JOB@/$$job/" \
	| sed "s/@TOPJOB@/$$topjob/" \
	| sed s/@SUFFIX@/$(SUFFIX)/ \
	| sed "s/@TESTSUMMARY@/$$summary/" \
	| sed "s/@PUBLISH_SUFFIX@/$(PUBLISH_SUFFIX)/" \
	> $@

# Top level rule for all files that start with 'gcc-'
ifneq (,$(filter benchmark,$(GCC_BUILD_OPTIONS)))
gcc-%/all.stamp: gcc-%/gcc.stamp gcc-%/benchmarks.stamp
	true
else
gcc-%/all.stamp: gcc-%/gcc.stamp
	true
endif

# Top level rule for all files that start with 'gdb-'
gdb-%/all.stamp: gdb-%/gdb.stamp
	true

# Top level rule for all files that start with 'llvm-'
llvm-%/all.stamp: llvm-%/llvm.stamp
	true

# Top level rule for all files that start with 'binutils-'
binutils-%/all.stamp: binutils-%/binutils.stamp
	true

# Top level rule for all files that start with 'gold-'
gold-%/all.stamp: gold-%/gold.stamp
	true

# Top level rule for all files that start with 'valgrind-'
valgrind-%/all.stamp: valgrind-%/valgrind.stamp
	true

# Top level rule for all files that start with 'meta-linaro-'
meta-linaro-%/all.stamp: meta-linaro-%/oecore.stamp
	true

# Top level rule for all files that start with 'openembedded-core-'
openembedded-core-%/all.stamp: openembedded-core-%/oecore.stamp
	true

# Top level rule for all files that start with 'bitbake-'
bitbake-%/all.stamp: bitbake-%/oecore.stamp
	true

# Top level rule for all files that start with 'oecore-'
oecore-%/all.stamp: oecore-%/oecore.stamp
	true

# Top level rule for all files that start with 'newlib-'
newlib-%/all.stamp: newlib-%/newlib.stamp
	true

# Top level rule for all files that start with 'qemu-'
qemu-%/all.stamp: qemu-%/qemu.stamp
	true

# Top level rule for all files that start with 'boot-wrapper-'
boot-wrapper-%/all.stamp: boot-wrapper-%/boot-wrapper.stamp
	true

# Top level rule for all files that start with 'libffi-'
libffi-%/all.stamp: libffi-%/libffi.stamp
	true

# Top level rule for all files that start with 'eglibc-'
eglibc-%/all.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/eglibc.mk B=$(@D)

# Top level rule for all files that start with 'cortex-strings-'
cortex-strings-%/all.stamp: cortex-strings-%/cortex-strings.stamp
	true

# Top level rule for all files that start with 'cortex-strings-'
linaro-toolchain-recipes-%/all.stamp: linaro-toolchain-recipes-%/linaro-toolchain-recipes.stamp
	true

# Top level rule for all files that start with 'crosstool-ng-'
crosstool-ng-%/all.stamp: crosstool-ng-%/crosstool-ng.stamp
	true

# Bounce to build the gcc benchmarks
benchmarks-gcc-%/all.stamp: gcc-%/benchmarks.stamp
	true

latest: latest/$(LATEST_TOOLCHAIN).stamp

# Check the latest reference toolchain is available.  Fetch from the
# build server, extract, and stamp
latest/gcc-%.stamp:
	rm -rf $(@D)
	mkdir -p $(@D)
	$(MAKE) -f $(LIBDIR)/fetch-binary.mk VERSION=$(LATEST_TOOLCHAIN) INTO=$(@D)
	tar xaf `ls -c $(@D)/$(LATEST_TOOLCHAIN)-$(MACHINE)-*tar*z | head -n1` -C $(@D) --strip-components=1
	rm -f $(@D)/*tar*z
	touch $@

# Special rule when testing against the host compiler
latest/gcc-native.stamp:
	rm -rf $(@D)
	mkdir -p $(@D)/bin
	touch $@

# Build gdb
%/gdb.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/gdb.mk B=$(@D)

# Build llvm
%/llvm.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/llvm.mk B=$(@D)

# Build binutils
%/binutils.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/binutils.mk B=$(@D)

# Build gold 
%/gold.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/gold.mk B=$(@D)

# Build binutils
%/valgrind.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/valgrind.mk B=$(@D)

# Build oecore against a particular GCC version
gcc-%/oecore.stamp:
	$(MAKE) -f $(LIBDIR)/oecore.mk B=$(@D) GCC_VERSION=$(@D)

# Build oecore against a particular meta-linaro version
meta-linaro-%/oecore.stamp:
	$(MAKE) -f $(LIBDIR)/oecore.mk B=$(@D) META_LINARO_VERSION=$(@D)

# Build oecore against a particular upstream snapshot version
openembedded-core-%/oecore.stamp:
	$(MAKE) -f $(LIBDIR)/oecore.mk B=$(@D) OECORE_VERSION=$(@D) GCC_FLAVOR="4.7%"

# Build oecore against a particular upstream snapshot version
bitbake-%/oecore.stamp:
	$(MAKE) -f $(LIBDIR)/oecore.mk B=$(@D) BITBAKE_VERSION=$(@D) GCC_FLAVOR="4.7%"

# Build oecore
%/oecore.stamp:
	$(MAKE) -f $(LIBDIR)/oecore.mk B=$(@D)

# Build newlib
%/newlib.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/newlib.mk B=$(@D)

# Build qemu
%/qemu.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/qemu.mk B=$(@D)

# Build boot-wrapper
%/boot-wrapper.stamp:
	$(MAKE) -f $(LIBDIR)/boot-wrapper.mk B=$(@D)

# Build libffi
%/libffi.stamp:
	$(MAKE) -f $(LIBDIR)/libffi.mk B=$(@D)

# Build cortex-strings
%/cortex-strings.stamp: latest/$(LATEST_TOOLCHAIN).stamp
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/runvariants.mk VARIANT_SRC=cross B=$(@D) AROUND=cortex-strings

# Build crosstool-ng.  The host compiler is fine.
%/crosstool-ng.stamp:
	$(MAKE) -f $(LIBDIR)/crosstool-ng.mk B=$(@D)

# Build linaro-toolchain-recipes
%/linaro-toolchain-recipes.stamp:
	$(LIBDIR)/runwith $(TOP)/latest \
	$(MAKE) -f $(LIBDIR)/linaro-toolchain-recipes.mk B=$(@D)

# Done as a for loop to allow the use of $BENCHMARKS and to get rid of
# the duplication of one rule per benchmark
gcc-%/benchmarks.stamp: gcc-%/gcc-binary.stamp gcc-%/host.txt
	for i in $(BENCHMARKS); do \
	$(LIBDIR)/runwith $(PWD)/$(@D)/gcc-binary \
	$(MAKE) -f $(LIBDIR)/runvariants.mk B=$(@D) AROUND=$$i; \
	done

# Done as a for loop to allow the use of $BENCHMARKS and to get rid of
# the duplication of one rule per benchmark
llvm-%/benchmarks.stamp: llvm-%/llvm-binary.stamp llvm-%/host.txt
	for i in $(BENCHMARKS); do \
	PATH=$(PWD)/$(@D)/llvm-binary/bin:$(PATH) \
	$(MAKE) -f $(LIBDIR)/runvariants.mk B=$(@D) AROUND=$$i CC=clang LD=gcc; \
	done

gcc-%/gcc-binary.stamp:
	mkdir -p $(@D)/gcc-binary
	$(MAKE) -f $(LIBDIR)/fetch-binary.mk VERSION=$(@D)
	tar xaf `ls -c $(@D)/$(@D)-$(MACHINE)-*tar*z | head -n1` -C $(@D)/gcc-binary --strip-components=1
	# Alias cc and c++
	rm -f $(@D)/gcc-binary/bin/cc
	rm -f $(@D)/gcc-binary/bin/c++
	cd $(@D)/gcc-binary/bin \
	&& ln -s gcc cc \
	&& ln -s g++ c++
	# Set up a trap for cc or specific gcc versions
	cd $(@D)/gcc-binary/bin \
	&& ln -s /bin/false gcc-4.4 \
	&& ln -s /bin/false gcc-4.5 \
	&& ln -s /bin/false g++-4.4 \
	&& ln -s /bin/false g++-4.5
	rm -f $(@D)/$(@D)-$(MACHINE)-*-cbuild*.tar*z
	touch $@

llvm-%/llvm-binary.stamp:
	mkdir -p $(@D)/llvm-binary
	rsync $(RSYNCFLAGS) -ti "$(BUILD_URL)/$(@D)/$(@D)-$(MACHINE)-*-cbuild*tar*z" $(@D)
	tar xaf `ls -c $(@D)/$(@D)-$(MACHINE)-*tar*z | head -n1` -C $(@D)/llvm-binary --strip-components=1
	touch $@

# Special case when running against the host toolchain
gcc-native/gcc-binary.stamp:
	mkdir -p $(@D)/gcc-binary/bin
	touch $@

# Libraries as tests
%/eglibc.stamp: %/gcc-binary.stamp
	$(LIBDIR)/runwith $(PWD)/$(@D)/gcc-binary \
	$(MAKE) -f $(LIBDIR)/eglibc.mk B=$(@D)
	touch $@

%/python.stamp: %/gcc-binary.stamp
	$(LIBDIR)/runwith $(PWD)/$(@D)/gcc-binary \
	$(MAKE) -f $(LIBDIR)/python.mk B=$(@D)
	touch $@

%/ubutest.stamp: %/gcc-binary.stamp
	$(LIBDIR)/runwith $(PWD)/$(@D)/gcc-binary \
	$(MAKE) -f $(LIBDIR)/ubutest.mk B=$(@D)
	touch $@

# GCC itself
%/gcc.stamp:
	$(MAKE) -f $(LIBDIR)/gcc.mk B=$(@D)
	touch $@

%/host.txt:
	mkdir -p $(@D)
	$(MAKE) -sf $(LIBDIR)/build.mk $@.spawn 2>&1 > $@

%/host.txt.spawn:
	echo date:
	date --rfc-3339=seconds -u
	echo
	echo uname:
	uname -a
	echo
	echo lsb_release:
	lsb_release -a
	echo
	echo /proc/version:
	cat /proc/version
	echo
	echo gcc:
	dpkg -s gcc | grep ^Version
	gcc --version
	echo as:
	dpkg -s binutils | grep ^Version
	as --version
	echo
	echo ldd:
	ldd --version
	echo
	echo free:
	free
	echo
	echo ulimit:
	bash -c "ulimit -a"
	echo
	echo cpuinfo:
	cat /proc/cpuinfo
	echo gdb:
	dpkg -s gdb | grep ^Version
	gdb --version
	echo gcc-binary:
	$(PWD)/$(@D)/gcc-binary/bin/gcc --version || true
	echo
	echo libc6:
	dpkg -s libc6 | grep ^Version
	echo PATH:
	echo $(PATH)
	echo
	echo df:
	-df -h / /scratch
	echo
	echo cpufreq-info:
	-cpufreq-info
	echo

.PRECIOUS:: \
	%/start.stamp \
	%/spawn.stamp \
	%/final.stamp \
	%/gcc.stamp \
	%/eglibc.stamp \
	%/bzip2bench.stamp \
	%/vorbisbench.stamp \
	%/benchnt.stamp \
	%/pybench.stamp \
	%/coremark.stamp \
	%/libavbench.stamp \
	%/publish.stamp \
	%/notify.stamp \
	gcc-%/gcc-binary.stamp \
	%/host.txt

check-benchmarks:
	mkdir -p local
	for i in $(BENCHMARKS); do \
	make -f ../../lib/$$i.mk BENCH_RUNS=2 PUBLISH_BASE=local; \
	done
