#!/usr/bin/env python
# Find the closest base filename.  Finds the one with the longest
# common prefix
#

import sys
import os.path
import pprint

def match(find, path):
    """Check a build against a queue, returning the common prefix if
    the needle starts with the queue name.

    The longest match is the closest.

    >>> match('foo', 'baz')
    >>> match('gcc-foo-bar', 'gcc')
    'gcc'
    >>> match('gcc-foo-bar', 'gcc-foo')
    'gcc-foo'
    >>> match('gcc-foo-bar', 'default')
    ''
    """
    base = os.path.basename(path)

    if find.startswith(base):
        return base
    elif base == 'default':
        return ''
    else:
        return None

def main():
    against = os.path.basename(sys.argv[1])
    prefixes = []

    for full in sys.argv[2:]:
        key = match(against, full)

        if key != None:
            prefixes.append((full, key))

    prefixes.sort(key=lambda x: -len(x[-1]))
    print prefixes[0][0]

def test():
    import doctest
    doctest.testmod()

if __name__ == '__main__':
    main()
