#!/bin/bash
#
# Pull the results from the cloud to the build server
#

set -e

dir=`mktemp -d`
trap "rm -rf $dir" EXIT

mkdir -p $dir/build
cd $dir

# Check what's there and fetch, aborting if it takes too long
timeout 600 s3cmd ls s3://cbuild \
    | awk '{print $4;}' \
    | grep 'logs.tar.xz$' \
    | xargs -r -t -P 3 -n 1 s3cmd get --skip-existing --no-progress

# Give up if nothing was found
ls *tar*z 2> /dev/null || exit 0

# Unpack
cd build
ls ../*tar*z | xargs -r -t -P 3 -n 1 tar xaf
cd ..

# Push across
rsync -rtiC build cbuild@orion:/var/www/ex.seabright.co.nz

# and remove
ls *tar*z | xargs -r -t -P 3 -n 1 -I {} s3cmd del s3://cbuild/{}
