#!/bin/sh

# Fail on any error
set -e

# Set the hostname
hostname=${1:-oort99}

echo $hostname > /etc/hostname
hostname $hostname
echo 127.0.0.1 $hostname proxy >> /etc/hosts

# Install the basic packages
apt-get update
apt-get install -yq puppet jed python-software-properties

# Add in the PPA
add-apt-repository ppa:linaro-toolchain-dev/build-deps
apt-get update

# Basics
apt-get build-dep -yq gcc binutils gdb
apt-get build-dep -yq gcc gcc-4.5 gdb eglibc ffmpeg llvm python libvorbis

# Needed for baremetal and graphite
apt-get install -yq binutils-arm-none-eabi libcloog-ppl-dev

# Needed for cbuild
apt-get install -yq time python-irclib wget gnupg ccrypt rsync coreutils sendemail bzr xdelta3 xz-utils unzip openssh-client gdb libio-socket-ssl-perl libnet-ssleay-perl

# Needed for cloud builds
apt-get install -yq polipo s3cmd

# Re-mount /mnt at /scratch
mnt=`df /mnt | tail -n1 | awk '{ print $1; }'`

umount /mnt || true
umount /scratch || true
mkdir -p /scratch
mount $mnt /scratch
chmod a+w /scratch

# Set up the cbuild user
cb=/home/cbuild

adduser --disabled-password --gecos cbuild cbuild
tar xaf *seed*.tar* -C /home/cbuild --strip-components=1

chown -R cbuild.cbuild $cb
chmod -R go-rwx $cb/.ssh

# Finally put /tmp in RAM for faster builds
mount -t tmpfs tmfs /tmp
