#!/bin/bash
#
# Runs a EC2 slave that builds cbuild jobs
#
# Usage: ec2slave.sh [hostname [ami [type]]]
#
# such as:
#  ec2slave.sh oort2 ami-689c6701 c1.medium
#  ec2slave.sh oort1 ami-40926929 c1.xlarge
#

api=http://ex.seabright.co.nz/helpers/scheduler/api
wget="wget --no-proxy -q"
this=ec2slave

hostname=${1:-oort1}
# Natty 64 bit instance store
ami=${2:-ami-1aad5273}
# 64 bit high-CPU
type=${3:-c1.xlarge}

region=us-east-1

# Bring in any local configuration
[ -f ~/.private/$this.rc ] && . ~/.private/$this.rc

dir=/tmp/$this/$hostname
mkdir -p $dir

set -e

function get_state {
    ec2din $instance > $dir/din
    state=`grep ^INSTANCE $dir/din | awk '{ print $6; }'`
    ip=`grep ^INSTANCE $dir/din | awk '{ print $4; }'`
    echo get_state on $instance: $state $ip
}

function start_instance {
    $wget -O - $api/update/$hostname/starting
    ec2-run-instances $ami -t $type --region $region -k $key > $dir/run
    instance=`grep ^INSTANCE $dir/run | awk '{ print $2; }'`

    trap "ec2kill $instance" EXIT

    get_state

    while [ "$state" != "running" ]; do
        sleep 5
        get_state
    done

    # Wait for Ubuntu to boot
    ssh -o StrictHostKeyChecking=no ubuntu@$ip uname -a || sleep 20
    ssh -o StrictHostKeyChecking=no ubuntu@$ip uname -a || sleep 20
    ssh -o StrictHostKeyChecking=no ubuntu@$ip uname -a

    echo Instance $instance started
}

function init_instance {
    scp $this-init.sh ~/.private/$this-*seed* ubuntu@$ip:~
    ssh ubuntu@$ip sudo bash $this-init.sh $hostname
    ssh -t cbuild@$ip bash init-cbuild.sh
}

function kill_old {
    if [ "$instance" != "" ]; then
        now=`date +%s`
        elapsed=$(($now-$used))

        if [ "$elapsed" -gt "600" ]; then
            echo Killing $instance due to inactivity
            ec2kill $instance
            instance=
        fi
    fi
}

function update_status {
    if [ "$instance" == "" ]; then
        $wget -O - $api/update/$hostname/lurking
    else
        $wget -O - $api/update/$hostname/idle
    fi
}

while true; do
    echo $hostname $instance `date`

    kill_old
    update_status

    # See if there's a job waiting
    next=`$wget -O - $api/peek/$hostname`

    case "$next" in
        False) sleep 60; continue ;;
    esac

    echo Running a job

    if [ "$instance" == "" ]; then
        echo Starting an instance
        start_instance
        init_instance
        used=`date +%s`
    fi

    ssh -o BatchMode=yes -t cbuild@$ip 'cd /scratch/cbuild/slave && ./run.sh'
    used=`date +%s`
done
