#!/bin/sh
#
# Update a git branch and snapshot the latest revision
#

set -e

dir=`mktemp -d`
trap "rm -rf $dir" EXIT

# Directory this script runs from
here=`dirname $0`

# Suffix for manual pulls
SUFFIX=${4:-}

# Pull a branch and tarball it up
branch=$1
head=$2
id=${3:-HEAD}

export GIT_DIR=$branch/.git
(cd $branch && git reset --hard -q && git pull -q)
latest=`git rev-parse $id`
short=`echo $latest | cut -c -7`
now=`date +%Y%m%d`

snapshots=$HOME/snapshots
tar=/tmp/$head$now+git$short$SUFFIX.tar
xdelta=$tar.xdelta3.xz

if ls $snapshots/$head*git$short*xdelta* > /dev/null 2>&1; then
    exit 0
fi

mkdir -p $snapshots
echo Exporting $short
git archive --format=tar --prefix=$head/ $id | tar xf - -C $dir
# Create the tarball
echo Creating $tar
(cd $dir && find $head -print0 | sort -z) | tar caf $tar -C $dir --no-recursion --null -T -
# Create the delta
closest=`$here/closest.py $tar $snapshots/base/*.tar`
echo Making the delta relative to $closest
xdelta3 -fe -s $closest $tar $tar.xdelta3
rm -f $tar
xz -f $tar.xdelta3
mv $tar.xdelta3.xz $snapshots
echo New snapshot `basename $tar` created
$here/spawn.sh $head `basename $tar .tar`
