#!/bin/bash
# Update a SVN branch and snapshot the latest revision
# Can also snapshot past revisions and add a per-user
# suffix

set -e

# Directory this script runs from
here=`dirname $0`

# Suffix for manual pulls
SUFFIX=${SUFFIX:-}

# Pull a branch and tarball it up
branch=$1
head=$2

svn up -q $branch

# needed for LLVM's multiple updates
if [[ $head = '' ]]; then
  echo "No tarball requested, update only, leaving."
  exit 0
fi

latest=`svn info $branch | awk '/Revision:/ { print $2 }'`
revno=${REVNO:-$latest}

snapshots=$HOME/snapshots
name=$2svn$revno$SUFFIX
tar=/tmp/$name.tar
xdelta=$snapshots/$name.tar.xdelta3.xz

if [ ! -f $xdelta ]; then
    echo Exporting and stamping $revno
    tmp=`mktemp -d`
    rm -rf $tmp/$head
    svn export $branch $tmp/$head
    REVNO=$revno $here/stamp_branch.sh $branch $tmp/$head

    # Create the tarball
    echo Creating $tar
    (cd $tmp && find $head -print0 | sort -z) | tar caf $tar -C $tmp --no-recursion --null -T -
    rm -rf $tmp
    # Create the delta
    closest=`$here/closest.py $tar $snapshots/base/*.tar`
    echo Making the delta relative to $closest
    xdelta3 -fe -s $closest $tar $tar.xdelta3
    rm -f $tar
    xz -f $tar.xdelta3
    mv $tar.xdelta3.xz $snapshots
    echo New snapshot $head $revno created
    $here/spawn.sh $head `basename $tar .tar`
fi

