"""Plots the backlog and utilisation level of a class of build
machines.
"""

import fileinput

import matplotlib
matplotlib.use('agg')

from pylab import *

def process(lines, name, matcher):
    start = float(lines[0][0])
    backlog = []
    running = []
    states = []
    hosts = {}

    for line in lines:
        time, type, host, arg = line[:4]

        if not matcher(host):
            continue

        offset = float(time) - start
        days = offset / (60*60*24)

        if type == 'backlog':
            if hosts[host] == 'reserved':
                # Reserved machines report as a zero backlog
                pass
            else:
                backlog.append((days, int(arg)))
        elif type == 'state':
            if arg != 'updating':
                states.append((host, arg, days))
                hosts[host] = arg

    # Scan through the states and remove the glitches
    merged = []

    for host in hosts:
        just = [x for x in states if x[0] == host]
        
        for i in range(len(just) - 1):
            this = just[i]
            next = just[i+1]
            elapsed = next[-1] - this[-1]

            if this[1] == 'running' and next[1] == 'idle' and elapsed < 5.0/60/60/24:
                # Drop this record
                pass
            else:
                merged.append(this)

    merged.sort(key=lambda x: x[-1])

    # Scan through and figure out the number of hosts running at any one time
    states = {}

    for row in merged:
        states[row[0]] = row[1]
        
        running.append((row[-1], sum(1 if x == 'running' else 0 for x in states.values())))

    backlog = array(backlog)
    running = array(running)

    clf()
    plot(backlog[:,0], backlog[:,1])
    scatter(backlog[:,0], backlog[:,1], label='backlog')

    plot(running[:,0], running[:,1], c='g', label='running')

    ylim(0, ylim()[1]+1)
    xlim(0, xlim()[1])
    xlabel('Time (days)')
    ylabel('Utilisation (#boards) & backlog (# jobs)')
    title('%s utilisation' % name)
    legend()
    grid()
    savefig('%s.png' % name)
    show()

def main():
    lines = list(fileinput.input())
    lines = [x.rstrip().split() for x in lines]
    
    process(lines, 'ursas', lambda x: x.startswith('ursa'))
#    process(lines, 'cloud', lambda x: x.startswith('oort'))

if __name__ == '__main__':
    main()
